//  Copyright 2019 Kakao Corp.
//
//  Licensed under the Apache License, Version 2.0 (the "License");
//  you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS,
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.

import UIKit
import Foundation
import KakaoSDKCommon
import KakaoSDKAuth
import KakaoSDKCert

import CryptoKit

public class CertDemoAppKey {
//    public static let sandboxAppKey1 = "a730e56eaa1d28d09128bc6febb0e7ac" //이용기관테스트앱 서명only sandbox
//    public static let sandboxAppKey2 = "93c0d6a6114ea95cbb507cfa6a7f6b17" //n동 4층 주식회사
    

    
//    sandbox
//    a730e56eaa1d28d09128bc6febb0e7ac (207933)
//    93c0d6a6114ea95cbb507cfa6a7f6b17 (208278)
//
//    cbt
//    28a88afd4adb1df9701d14e8e615ffaf (644363)
//    ca0baf00ead5aeda472f045e65ce2c11 (865425)
//
//    prod
//    28a88afd4adb1df9701d14e8e615ffaf (644363)
//    ca0baf00ead5aeda472f045e65ce2c11 (865425)
    
    
    var _appKeys: [String]
    
    public init(appKeys: [String] = ["28a88afd4adb1df9701d14e8e615ffaf","ca0baf00ead5aeda472f045e65ce2c11"]) {
        _appKeys = appKeys
    }
    
    public func appKeys() -> [String] {
        return _appKeys
    }
    
}


///:nodoc: 이용기관 테스트용 mock-server
public class CertDemoHost {
    var _domain: String
    
    public init(domain: String = "zert-mock.dev.onkakao.net") {
        _domain = domain
    }
    
    public func domain() -> String {
        return _domain
    }
}

public class CertDemoUrl {
    //kakaocert k2220/k3220 demo(임시 이용기관)
//    public static let demoSign = "/sign"
//    public static let demoVerify = "/verify"
//    public static let demoReducedSign = "/reduction-sign"
//    public static let demoSignTest = "/reduction-sign/temp"
    
    public static let demoSign = "/v1/%@/sign"
    public static let demoVerify = "/v1/%@/verify"
    public static let demoReducedSign = "/v1/%@/reduction-sign"
    public static let demoSignTest = "/v1/%@/reduction-sign/temp"
    
    public static func pathCompose(certType:String, path:String) -> String {
        return String(format: path, certType)
    }
}

///nodoc:
import Alamofire

///nodoc: //demo 서버 api
public class CertDemoApi {
    public static let shared = CertDemoApi()
    
    static let txIdInfoKey = "com.kakao.sdk.txIdInfo"
//    static let txIdKey = "com.kakao.sdk.k2220.txId"
//    private var _txId: String? = nil
    private var _txIdInfo : [String:String]
    
    public var demoHost: CertDemoHost
    public var demoAppKey: CertDemoAppKey
    
    public init() {
//        _txId = UserDefaults.standard.string(forKey: CertDemoApi.txIdKey)
        if let txIdInfo = UserDefaults.standard.value(forKey:CertDemoApi.txIdInfoKey) as? [String: String] {
            _txIdInfo = txIdInfo
        }
        else {
            _txIdInfo = [:]
        }
        
        demoHost = CertDemoHost()
        demoAppKey = CertDemoAppKey()
    }
}

extension CertDemoApi {
    public func txId(certType:CertType) -> String? {
        return _txIdInfo[certType.rawValue]
    }
    
    public func saveTxId(_ txId: String, certType:CertType) {
        _txIdInfo[certType.rawValue] = txId
        UserDefaults.standard.set(_txIdInfo, forKey:CertDemoApi.txIdInfoKey)
        UserDefaults.standard.synchronize()
    }
    
    public func deleteTxId(certType:CertType) {
        _txIdInfo.removeValue(forKey: certType.rawValue)
        UserDefaults.standard.set(_txIdInfo, forKey:CertDemoApi.txIdInfoKey)
        UserDefaults.standard.synchronize()
    }
    
    public func deleteTxIdInfo() {
        UserDefaults.standard.removeObject(forKey: CertDemoApi.txIdInfoKey)
        UserDefaults.standard.synchronize()
        
        _txIdInfo.removeAll()
    }
    
    
//    public func txId() -> String? {
//        return _txId
//    }
//
//    public func saveTxId(_ txId: String?) {
//        _txId = txId
//        UserDefaults.standard.setValue(_txId, forKey:CertDemoApi.txIdKey)
//        UserDefaults.standard.synchronize()
//
//    }
//
//    public func deleteTxId() {
//        UserDefaults.standard.removeObject(forKey: CertDemoApi.txIdKey)
//        _txId = nil
//    }
}
   
extension CertDemoApi {
    ///:nodoc:
    public func demoSign(certType: CertType,
                         base64encodedDerPublicKey:String? = nil,
                         returnUrl:String? = nil,
                         validateAppKey:String? = nil,
                         targetAppKey:String? = nil,
                         completion: @escaping (String?, Error?) -> Void) {
        
        
        guard let base64encodedDerPublicKey = base64encodedDerPublicKey else {
            completion(nil, CertDemoError(msg:"base64encodedDerPublicKey is nil"))
            return
        }

        var param: [String:Any] = [:]
        param["public_key"] = base64encodedDerPublicKey

        
        if certType == .K3220 {
            guard let returnUrl = returnUrl else {
                completion(nil, CertDemoError(msg:"returnUrl is nil"))
                return
            }
            param["return_url"] = returnUrl
            
            guard let targetAppKey = targetAppKey else {
                completion(nil, CertDemoError(msg:"targetAppKey is nil"))
                return
            }
            param["target_app_key"] = targetAppKey
            
            guard let validateAppKey = validateAppKey else {
                completion(nil, CertDemoError(msg:"validateAppKey is nil"))
                return
            }
            param["validate_app_key"] = validateAppKey
        }
        let url = "https://\(CertDemoApi.shared.demoHost.domain())\(CertDemoUrl.pathCompose(certType:certType.rawValue, path:CertDemoUrl.demoSign))"
        
        //TODO:릴리즈시 print로 교체
        SdkLog.i("request: \n method: \(HTTPMethod.post)\n url:\(url)\n parameters: \(String(describing: param)) \n\n")

        AF.request(url, method: .post, parameters: param)
          .responseData(completionHandler: { response in
              
              //TODO:릴리즈시 print로 교체
              SdkLog.i("response:\n \(String(describing: response))\n\n" )
              
              if let error = response.error {
                  completion(nil, error)
              }
              else {
                  guard let data = response.data else {
                      completion(nil, CertDemoError(msg:"data is nil"))
                      return
                  }
                  
                  if let error = CertDemoError(data: data) {
                      completion(nil, error)
                  }
                  else {
                      if let certResponse = try? SdkJSONDecoder.custom.decode(CertDemoResponse.self, from: data) {
                          print("response: \(certResponse)")
                          completion(certResponse.txId, nil)
                      }
                  }
              }
          })
    }

    ///:nodoc:
    public func demoVerify(certType:CertType,
                           txId: String,
                           appUserId: Int64? = nil,
                           targetAppKey: String? = nil,
                           completion: @escaping (CertDemoResponse?, Error?) -> Void) {
        
        var param: [String:Any] = [:]
        param["tx_id"] = txId

        if certType == .K3220 {
            guard let targetAppKey = targetAppKey else {
                completion(nil, CertDemoError(msg:"targetAppKey is nil"))
                return
            }
            param["target_app_key"] = targetAppKey
        }
        else { //k2220
            guard let appUserId = appUserId else {
                completion(nil, CertDemoError(msg:"appUserId is nil"))
                return
            }
            param["app_user_id"] = appUserId
        }
        let url = "https://\(CertDemoApi.shared.demoHost.domain())\(CertDemoUrl.pathCompose(certType:certType.rawValue, path:CertDemoUrl.demoVerify))"
        
        //TODO:릴리즈시 print로 교체
        SdkLog.i("request: \n method: \(HTTPMethod.post)\n url:\(url)\n parameters: \(String(describing: param)) \n\n")

        AF.request(url, method: .post, parameters: param)
          .responseData(completionHandler: { response in
              
              //TODO:릴리즈시 print로 교체
              SdkLog.i("response:\n \(String(describing: response))\n\n" )
              
              if let error = response.error {
                  completion(nil, error)
              }
              else {
                  guard let data = response.data else {
                      completion(nil, CertDemoError(msg:"data is nil"))
                      return
                  }
                  
                  if let error = CertDemoError(data: data) {
                      completion(nil, error)
                  }
                  else {
                      if let certResponse = try? SdkJSONDecoder.custom.decode(CertDemoResponse.self, from: data) {
                          print("response: \(certResponse)")
                          completion(certResponse, nil)
                      }
                  }
              }
          })
    }

    ///:nodoc:
    public func demoReducedSign(certType:CertType,
                                txId:String,
                                data:String,
                                signature:String,
                                completion: @escaping (CertDemoResponse?, Error?) -> Void) {
        let param: [String:String] = [
            "tx_id":txId,
            "data": data,
            "signature":signature
        ]
        let url = "https://\(CertDemoApi.shared.demoHost.domain())\(CertDemoUrl.pathCompose(certType:certType.rawValue, path:CertDemoUrl.demoReducedSign))"
        
        //TODO:릴리즈시 print로 교체
        SdkLog.i("request: \n method: \(HTTPMethod.post)\n url:\(url)\n parameters: \(String(describing: param)) \n\n")

        AF.request(url, method: .post, parameters: param)
          .responseData(completionHandler: { response in
              
              //TODO:릴리즈시 print로 교체
              SdkLog.i("response:\n \(String(describing: response))\n\n" )
              
              if let error = response.error {
                  completion(nil, error)
              }
              else {
                  guard let data = response.data else {
                      completion(nil, CertDemoError(msg:"data is nil"))
                      return
                  }
                  
                  if let error = CertDemoError(data: data) {
                      completion(nil, error)
                  }
                  else {
                      if let certResponse = try? SdkJSONDecoder.custom.decode(CertDemoResponse.self, from: data) {
                          print("response: \(certResponse)")
                          completion(certResponse, nil)
                      }
                  }
              }
          })
    }
}

////k3220
//extension CertDemoApi {
//    public func demoLogin(base64encodedDerPublicKey:String? = nil,
//                          returnUrl:String? = nil,
//                          certType: CertProductCode? = nil,
//                          completion: @escaping (String?, Error?) -> Void) {
//        guard let base64encodedDerPublicKey = base64encodedDerPublicKey else {
//            completion(nil, CertDemoError(msg:"publicKey is nil"))
//            return
//        }
//        guard let returnUrl = returnUrl else {
//            completion(nil, CertDemoError(msg:"returnUrl is nil"))
//            return
//        }
//
//        let param: [String:String] = [
//            "public_key":base64encodedDerPublicKey,
//            "return_nil":returnUrl
//        ]
//
//        //TODO: k3220 url 변경필요
//        let url = "https://\(CertDemoApi.shared.demoHost.domain())\(Paths.demoLogin)"
//        
//        //TODO:릴리즈시 print로 교체
//        SdkLog.i("request: \n method: \(HTTPMethod.post)\n url:\(url)\n parameters: \(String(describing: param)) \n\n")
//
//        AF.request(url, method: .post, parameters: param)
//          .responseData(completionHandler: { response in
//              
//              //TODO:릴리즈시 print로 교체
//              SdkLog.i("response:\n \(String(describing: response))\n\n" )
//              
//              if let error = response.error {
//                  completion(nil, error)
//              }
//              else {
//                  guard let data = response.data else {
//                      completion(nil, CertDemoError(msg:"data is nil"))
//                      return
//                  }
//                  
//                  if let error = CertDemoError(data: data) {
//                      completion(nil, error)
//                  }
//                  else {
//                      if let certResponse = try? SdkJSONDecoder.custom.decode(CertDemoResponse.self, from: data) {
//                          print("response: \(certResponse)")
//                          completion(certResponse.txId, nil)
//                      }
//                  }
//              }
//          })
//    }
//}

public struct CertDemoResponse : Codable {
    public let code: String?
    public let msg: String?
    public let txId: String?
}

public struct CertDemoError : Error {
    public let code: String?
    public let msg: String?
    
    public init?(data:Data) {
        guard let certDemoResponse = try? SdkJSONDecoder.custom.decode(CertDemoResponse.self, from: data) else {
            self.code = "E----"
            self.msg = "can't parse response"
            return
        }
        if certDemoResponse.code == "E0000" { return nil }
        
        self.code = certDemoResponse.code
        self.msg = certDemoResponse.msg
    }
    
    public init(code:String? = nil, msg:String? = nil) {
        self.code = code
        self.msg = msg ?? "unknown"
    }
}
