//  Copyright 2019 Kakao Corp.
//
//  Licensed under the Apache License, Version 2.0 (the "License");
//  you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS,
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.

import UIKit
import Foundation
import KakaoSDKCommon
import KakaoSDKAuth
import KakaoSDKCert

import RxSwift

extension CertDemoApi: ReactiveCompatible {}

extension Reactive where Base: CertDemoApi  {
    //Rx CertDemoApi
    ///:nodoc:
    public func demoSign(certType: CertType,
                         base64encodedDerPublicKey:String? = nil,
                         returnUrl:String? = nil,
                         validateAppKey:String? = nil,
                         targetAppKey:String? = nil) -> Observable<String> {
        return Observable<String>.create { observer in
            CertDemoApi.shared.demoSign(certType: certType,
                                        base64encodedDerPublicKey: base64encodedDerPublicKey,
                                        returnUrl: returnUrl,
                                        validateAppKey: validateAppKey,
                                        targetAppKey: targetAppKey) { txId, error in
                if let error = error {
                    observer.onError(error)
                }
                else {
                    if let txId = txId {
                        observer.onNext(txId)
                    }
                    else {
                        observer.onError(SdkError(reason: .IllegalState, message: "response is nil"))
                    }
                }
            }            
            return Disposables.create()
        }
    }
    
    public func demoVerify(certType:CertType,
                           txId: String,
                           appUserId: Int64? = nil,
                           targetAppKey: String? = nil) -> Observable<CertDemoResponse> {
        return Observable<CertDemoResponse>.create { observer in
            CertDemoApi.shared.demoVerify(certType: certType,
                                          txId: txId,
                                          appUserId: appUserId,
                                          targetAppKey: targetAppKey) { certDemoResponse, error in
                if let error = error {
                    observer.onError(error)
                }
                else {
                    if let certDemoResponse = certDemoResponse {
                        observer.onNext(certDemoResponse)
                    }
                    else {
                        observer.onError(SdkError(reason: .IllegalState, message: "response is nil"))
                    }
                }
            }
            
            return Disposables.create()
        }
    }
    
    public func demoReducedSign(certType:CertType,
                                txId:String,
                                data:String,
                                signature:String) -> Observable<CertDemoResponse> {
        return Observable<CertDemoResponse>.create { observer in
            CertDemoApi.shared.demoReducedSign(certType: certType,
                                               txId: txId,
                                               data: data,
                                               signature: signature) { certDemoResponse, error in
                if let error = error {
                    observer.onError(error)
                }
                else {
                    if let certDemoResponse = certDemoResponse {
                        observer.onNext(certDemoResponse)
                    }
                    else {
                        observer.onError(SdkError(reason: .IllegalState, message: "response is nil"))
                    }
                }
            }
            
            return Disposables.create()
        }
    }
}

